﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using HttpSignatures;
using NUnit.Framework;

namespace HttpSignatureTests
{
    [TestFixture()]
    public class ApplySignatureTest
    {
        [Test]
        public void ApplySignatureToIRequest()
        {
            var signer = new HttpSigner(new AuthorizationParser(), new HttpSignatureStringExtractor());

            var request = new Request();
            request.Path = "/hello-world";
            request.Method = HttpMethod.Post;
            request.SetHeader("host", "example.com");
            request.SetHeader("date", DateTime.UtcNow.ToString("r"));

            var spec = new SignatureSpecification()
            {
                Algorithm = "hmac-sha256",
                Headers = new string[] { "host", "(request-target)" },
                KeyId = Fixture.KeyId
            };
            signer.Sign(request, spec, Fixture.KeyId, Fixture.KeyBase64);
            var keyStore = new KeyStore(new Dictionary<string, string>()
            {
                {Fixture.KeyId, Fixture.KeyBase64}
            });
            var signature = signer.Signature(request, spec, keyStore);
            Assert.True(signature.Valid);
            Console.WriteLine("hahha~~~~1");
        }

        [Test]
        public void SignRequestTest()
        {
            var uri = "/foo/Bar";

            String authorization = "Signature keyId=\"hmac-key-1\",algorithm=\"hmac-sha256\",headers=\"content-length host date (request-target)\",signature=\"SSyfl2vWll/QzOVHIjxnriuHGrMTX5i/0KIKCS0ydo8=\"";
            String keyId = "hmac-key-1";
            String privateKey = "6Jj3c7lQr6dhDf4oMsYnTfrjFnwezP4GzqHlqr2heyw=";

            var spec = new SignatureSpecification()
            {
                Algorithm = "hmac-sha256",
                Headers = new string[] { "content-length", "host", "date", "(request-target)" },
                KeyId = keyId
            };

            var request = new Request();
            request.Path = uri;
            request.Method = HttpMethod.Get;
            request.SetHeader("content-length", "18");
            request.SetHeader("host", "example.org");
            request.SetHeader("date", "Tue, 07 Jun 2014 20:51:35 GMT");

            var signer = new HttpSigner(new AuthorizationParser(), new HttpSignatureStringExtractor());
            signer.Sign(request, spec, keyId, privateKey);
            Console.WriteLine(request.GetHeader("Authorization"));
            Assert.AreEqual(authorization, request.GetHeader("Authorization"));

            var keyStore = new KeyStore(new Dictionary<string, string>()
            {
                {keyId, privateKey}
            });
            var signature = signer.Signature(request, spec, keyStore);
            Assert.True(signature.Valid);
            Console.WriteLine(signature.Headers);

            string s = signer.CalculateSignature(request, spec, privateKey);
            Console.WriteLine(s);
            Assert.AreEqual("SSyfl2vWll/QzOVHIjxnriuHGrMTX5i/0KIKCS0ydo8=", s);
        }

        [Test]
        public void SignRequestTest2()
        {
            String authorization = "Signature keyId=\"hmac-key-1\",algorithm=\"hmac-sha256\",headers=\"content-length host date (request-target)\",signature=\"yT/NrPI9mKB5R7FTLRyFWvB+QLQOEAvbGmauC0tI+Jg=\"";
            String keyId = "hmac-key-1";        // accessKey
            String privateKey = "don't tell";   // secretKey

            // 定义要加签的headers和算法
            var spec = new SignatureSpecification()
            {
                Algorithm = "hmac-sha256",
                Headers = new string[] { "content-length", "host", "date", "(request-target)" },
                KeyId = keyId
            };

            // http 请求
            var request = new Request();
            request.Path = "/foo/Bar";
            request.Method = HttpMethod.Get;
            request.SetHeader("content-length", "18");
            request.SetHeader("host", "example.org");
            request.SetHeader("date", "Tue, 07 Jun 2014 20:51:35 GMT");

            // 签名计算，计算之后，签名信息放在request的 header["Authorization"] 里
            var signer = new HttpSigner(new AuthorizationParser(), new HttpSignatureStringExtractor());
            signer.Sign(request, spec, keyId, privateKey);
            Console.WriteLine(request.GetHeader("Authorization"));
            Assert.AreEqual(authorization, request.GetHeader("Authorization"));

            // 独立的签名计算方法
            string s = signer.CalculateSignature(request, spec, privateKey);
            Console.WriteLine(s);
            Assert.AreEqual("yT/NrPI9mKB5R7FTLRyFWvB+QLQOEAvbGmauC0tI+Jg=", s);
        }

        [Test]
        public void SignReponseTest()
        {
            String authorization = "Signature keyId=\"keyid\",algorithm=\"hmac-sha256\",headers=\"host date content-type\",signature=\"YlTvvCwETlw/Xa5z+y7Dzk70MzE6IITZ+VDtBrQ2Ca8=\"";
            String keyId = "keyid";
            String privateKey = "secret";

            var spec = new SignatureSpecification()
            {
                Algorithm = "hmac-sha256",
                Headers = new string[] { "host", "date", "content-type" },
                KeyId = keyId
            };

            var request = new Request();
            request.Path = "/foo/Bar";
            request.Method = HttpMethod.Post;
            request.SetHeader("content-type", "application/json");
            request.SetHeader("host", "test");
            request.SetHeader("date", "Tue, 07 Jun 2014 20:51:35 GMT");

            var signer = new HttpSigner(new AuthorizationParser(), new HttpSignatureStringExtractor());
            signer.Sign(request, spec, keyId, privateKey);
            Console.WriteLine(request.GetHeader("Authorization"));
            Assert.AreEqual(authorization, request.GetHeader("Authorization"));

            string s = signer.CalculateSignature(request, spec, privateKey);
            Console.WriteLine(s);
            Assert.AreEqual("YlTvvCwETlw/Xa5z+y7Dzk70MzE6IITZ+VDtBrQ2Ca8=", s);
        }
    }
}
